function phaseout = newMusclesStimOptContinuous(input)

% Define persistent variables so that splines are not conducted on each
% continuous function call
persistent mkrData grfr grfl baseNeuralCommands_r baseNeuralCommands_l idTorques mcr mcl q qppp handLoadsOptR handLoadsOptL %idData

% rescale everything back to original
nPts = length(input.phase.time);
onesCol = ones(nPts,1);

% inputorig = input;

% input.phase.time = (input.phase.time)*(input.auxdata.tMax-input.auxdata.tMin)+(input.auxdata.tMax+input.auxdata.tMin)/2;
input.phase.time = input.phase.time*input.auxdata.timeScale;
input.phase.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;
input.phase.control = (input.phase.control).*(onesCol*(input.auxdata.controlMax-input.auxdata.controlMin))+onesCol*(input.auxdata.controlMax+input.auxdata.controlMin)/2;
% input.phase.parameter = (input.phase.parameter(1,:)).*((input.auxdata.paramMax-input.auxdata.paramMin))+(input.auxdata.paramMax+input.auxdata.paramMin)/2;
input.phase.parameter = input.phase.parameter(1,:).*input.auxdata.paramScale;

%% Load parameters from the auxilary data
tScale = input.auxdata.tScale;

mu_s = 0; % Coefficient of static friction for foot-ground contact
mu_d = 1;%input.phase.parameter(1,9);%input.auxdata.mu;%parameters(1,1); % Coefficient of dynamic friction
mu_v = 0; % Coefficient of viscous friction
kVal = input.auxdata.kVal;%input.auxdata.Kval; % Spring stiffnesses for foot ground contact
cVal = input.auxdata.cVal; % Spring damping
yVal = input.auxdata.yVal;
springMat = input.auxdata.springMat;
springBodyMat = input.auxdata.springBodyMat;
numSpringsBody = input.auxdata.numSpringsBody;

springMat(7:6+input.auxdata.numSprings/2,2) = springMat(7:6+input.auxdata.numSprings/2,2)+yVal(1);
springMat(7+input.auxdata.numSprings/2:end,2) = springMat(7+input.auxdata.numSprings/2:end,2)+yVal(2);

% Transition velocity between static and dynamic friction
latchvel = input.auxdata.latchvel;

muscParams.lmo = input.auxdata.lmo;
muscParams.lts = input.auxdata.lts;
muscParams.fMax = input.auxdata.fMax;
muscParams.pennAngle = input.auxdata.pennAngle;
muscParams.etmCoefs = input.auxdata.etmCoefs;
muscParams.nPts = nPts;
muscParams.muscRef = input.auxdata.muscRef;
muscParams.tAct = [input.auxdata.tAct input.auxdata.tAct];
muscParams.aNonlin = [input.auxdata.aNonlin input.auxdata.aNonlin];


%% Initialize Variables

t = input.phase(1).time;

if ~exist('mkrData','var')
    q = fnval(input.auxdata.spq,t)';
    qppp = fnval(input.auxdata.spqppp,t)';
    mkrData = fnval(input.auxdata.spMkrData,t)';
    grfr = fnval(input.auxdata.spGRFR,t)';
    grfl = fnval(input.auxdata.spGRFL,t)';
%     idData = fnval(input.auxdata.spID,t)';
    idTorques = fnval(input.auxdata.spTorques,t)';
    mcr = fnval(input.auxdata.spMcr,t)';
    mcl = fnval(input.auxdata.spMcl,t)';
    baseNeuralCommands_r = fnval(input.auxdata.spSynergyCommands_r,input.auxdata.time(end)*t/t(end))';
    baseNeuralCommands_l = fnval(input.auxdata.spSynergyCommands_l,input.auxdata.time(end)*t/t(end))';
    handLoadsOptR = fnval(input.auxdata.spHandLoadsR,input.auxdata.time(end)*t/t(end))';
    handLoadsOptL = fnval(input.auxdata.spHandLoadsL,input.auxdata.time(end)*t/t(end))';
elseif size(mkrData,1) ~= nPts
    q = fnval(input.auxdata.spq,t)';
    qppp = fnval(input.auxdata.spqppp,t)';
    mkrData = fnval(input.auxdata.spMkrData,t)';
    grfr = fnval(input.auxdata.spGRFR,t)';
    grfl = fnval(input.auxdata.spGRFL,t)';
%     idData = fnval(input.auxdata.spID,t)';
    idTorques = fnval(input.auxdata.spTorques,t)';
    mcr = fnval(input.auxdata.spMcr,t)';
    mcl = fnval(input.auxdata.spMcl,t)';
    baseNeuralCommands_r = fnval(input.auxdata.spSynergyCommands_r,input.auxdata.time(end)*t/t(end))';
    baseNeuralCommands_l = fnval(input.auxdata.spSynergyCommands_l,input.auxdata.time(end)*t/t(end))';
    handLoadsOptR = fnval(input.auxdata.spHandLoadsR,input.auxdata.time(end)*t/t(end))';
    handLoadsOptL = fnval(input.auxdata.spHandLoadsL,input.auxdata.time(end)*t/t(end))';
end

%% Kinematics

% Set states
x = input.phase(1).state(:,1:31);
xp = input.phase(1).state(:,32:62);

% Calculate spring positions and velocities
[springPos, springVels] = opensimPointKin_v3(t,[x xp],springMat',springBodyMat);

% Store location of heel and toes body for tranferring GRFs
BodyPositions = springPos(:,1:12);
% mcr = SpringPos(:,13:15);
% mcl = SpringPos(:,16:18);
springPos(:,1:18) = [];
springVels(:,1:18) = [];

mcr(:,2) = 0;
mcl(:,2) = 0;

%% Calculate Ground Reaction Forces

% Calculate spring forces and moments about electrical centers
[footGRFsRMC, footGRFsLMC] = calcGroundReactions_v3(springPos, springVels, ...
    numSpringsBody, mu_s, mu_d, mu_v, kVal, cVal, mcr, mcl, latchvel, tScale,input.auxdata.beltSpeed);

% Transfer moments to heel and toe origin for input into full body model
% This is necessary because the contact model ground reactions are
% calculated about some point in the room (force plate electrical centers),  
% and the model applies the loads at the origin of the foot and toes body
footGRFsR = footGRFsRMC;
footGRFsL = footGRFsLMC;

% Transfer to heel
footGRFsR(:,7:9) = cross(mcr-BodyPositions(:,1:3),footGRFsRMC(:,1:3),2)+footGRFsRMC(:,7:9);
footGRFsL(:,7:9) = cross(mcl-BodyPositions(:,7:9),footGRFsLMC(:,1:3),2)+footGRFsLMC(:,7:9);

% Transfer to toe
footGRFsR(:,10:12) = cross(mcr-BodyPositions(:,4:6),footGRFsRMC(:,4:6),2)+footGRFsRMC(:,10:12);
footGRFsL(:,10:12) = cross(mcl-BodyPositions(:,10:12),footGRFsLMC(:,4:6),2)+footGRFsLMC(:,10:12);

% footGRFsR = zeros(nPts,12);
% footGRFsL = zeros(nPts,12);
% footGRFsR(:,1:3) = grfr(:,1:3);
% footGRFsL(:,1:3) = grfl(:,1:3);
% 
% footGRFsR(:,7:9) = cross(mcr-BodyPositions(:,1:3),grfr(:,1:3),2)+grfr(:,4:6);
% footGRFsL(:,7:9) = cross(mcl-BodyPositions(:,7:9),grfl(:,1:3),2)+grfl(:,4:6);



%% Calculate activations change from excitations
stimIndex1 = input.auxdata.stimIndex1;
stimIndex2 = input.auxdata.stimIndex2;

stimMag1 = input.phase.parameter(1);
stimMag2 = input.phase.parameter(2);
stimTOn1 = input.phase.parameter(3);
stimTOn2 = input.phase.parameter(4);
stimTOff1 = stimTOn1 + input.phase.parameter(5);
stimTOff2 = stimTOn2 + input.phase.parameter(6);
% gasStimMag = input.phase.parameter(1);
% taStimMag = input.phase.parameter(2);
% gasStimTOn = input.phase.parameter(3);
% taStimTOn = input.auxdata.taStimTOn;
% gasStimTOff = gasStimTOn + input.phase.parameter(4);
% taStimTOff = taStimTOn + 0.6;
% gasStimTOn = input.auxdata.gasStimTOn;
% taStimTOn = input.phase.parameter(3);
% gasStimTOff = gasStimTOn + 0.06;
% taStimTOff = taStimTOn + input.phase.parameter(4);
% gasStimMag = input.phase.parameter(1);
% taStimMag = input.phase.parameter(2);
% gasStimTOn = 0.4; %input.auxdata.iii;
% taStimTOn = input.auxdata.taStimTOn;
% gasStimTOff = gasStimTOn + input.phase.parameter(3);
% taStimTOff = taStimTOn + input.phase.parameter(4);
% gasStimTOff = gasStimTOn + input.auxdata.optStimParams(5);
% taStimTOff = taStimTOn + input.auxdata.optStimParams(6);

neuralCommands = input.phase.control(:,32:41);
neuralCommands_r = neuralCommands(:,1:5);
neuralCommands_l = neuralCommands(:,6:10);
synergyWeights_r = input.auxdata.synergyWeights_r;
synergyWeights_l = input.auxdata.synergyWeights_l; %reshape(input.phase.parameter(1,:),10,35);

activations(:,1:36) = neuralCommands_r*synergyWeights_r;
activations(:,37:72) = neuralCommands_l*synergyWeights_l;
% activations(:,1:36) = baseNeuralCommands_r*synergyWeights_r;
% activations(:,37:72) = baseNeuralCommands_l*synergyWeights_l;

% Add activations from stimulation
% gasStimAct = gasStimMag*stimActivationOverflow(t,input.auxdata.gasStim_optParams,gasStimTOn,gasStimTOff);
% taStimAct = taStimMag*stimActivationOverflow(t,input.auxdata.taStim_optParams,taStimTOn,taStimTOff);
% gasStimAct = gasStimMag*freglyStimActivationOverflow(t,input.auxdata.tAct(26),4*input.auxdata.tAct(26),gasStimTOn,gasStimTOff);
% taStimAct = taStimMag*freglyStimActivationOverflow(t,input.auxdata.tAct(27),4*input.auxdata.tAct(27),taStimTOn,taStimTOff);
stimAct1 = stimMag1*stimActivationOverflow_simple(t,input.auxdata.stimOptParams_simple{stimIndex1},stimTOn1,stimTOff1);
stimAct2 = stimMag2*stimActivationOverflow_simple(t,input.auxdata.stimOptParams_simple{stimIndex2},stimTOn2,stimTOff2);

actsWithStim = activations;
actsWithStim(:,stimIndex1) = activations(:,stimIndex1) + stimAct1;
actsWithStim(:,stimIndex2) = activations(:,stimIndex2) + stimAct2;

% activations(activations<0) = 0;
muscParams.activations = actsWithStim;


%% Calculate moments from EMG Data

muscParams.EtMMats = BuildEtMMatrices(x(:,[1:6 8:13]+6),xp(:,[1:6 8:13]+6));
[etmMoments,muscForce,v_mt] = calcMoments(muscParams);


%% Dynamics

% Extract for input into full body dynamic model
torques = zeros(nPts,25);%Set to zero because they will be determined by inverse dynamics

% Formulate inputs into opensim (states and controls w/ GRFs)
states = [x xp];
% appliedLoads = [torques footGRFsR footGRFsL handLoadsR handLoadsL];
appliedLoads = [torques footGRFsR footGRFsL handLoadsOptR handLoadsOptL];
% xpp = input.phase(1).control(:,1:29);

states(isnan(states)) = 0;
states(isinf(states)) = 0;

% Set accelerations
xpp = input.phase.state(:,63:end);
% Set jerk
xppp = input.phase.control(:,1:input.auxdata.nJerkControls);


% Calculate state derivatives and marker locations
% [residualLoads, mkrPos] = Control_OpenSimMPI_muscle_ID([x xp], xpp, appliedLoads,t,[1 0 0]);
[residualLoads, mkrPos] = opensimID_OMP(t,[x xp], xpp, appliedLoads,[1 1 0]);
torques = residualLoads(:,7:end);

% Create new reordered marker matrix
mkrPosReorder = zeros(size(mkrPos,1),size(mkrPos,2)*size(mkrPos,3));
for iii = 1:size(mkrPos,2)
    mkrPosReorder(:,(iii-1)*3+1) = squeeze(mkrPos(:,iii,1));
    mkrPosReorder(:,(iii-1)*3+2) = squeeze(mkrPos(:,iii,2));
    mkrPosReorder(:,(iii-1)*3+3) = squeeze(mkrPos(:,iii,3));
end

% Set rescaled derivative of state vector
phaseout(1).dynamics = (input.auxdata.tMax-input.auxdata.tMin)*([xp xpp xppp])./(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin));



%% Cost Function

% Output GRFs split between segments used for creating plots after
% simulation finishes
phaseout.GRFsSplit = [footGRFsRMC footGRFsLMC];
% phaseout.GRFsSplit = [grfr grfl];

% Calculate net GRFs at the electrical center for use in objective function
footGroundReactionsRMC = footGRFsRMC(:,[1:3 7:9])+footGRFsRMC(:,[4:6 10:12]);
footGroundReactionsLMC = footGRFsLMC(:,[1:3 7:9])+footGRFsLMC(:,[4:6 10:12]);

% propulsiveForce = [footGroundReactionsRMC(:,1).*(tanh(footGroundReactionsRMC(:,1)) + 1)/2,...
%     footGroundReactionsLMC(:,1).*(tanh(footGroundReactionsLMC(:,1)) + 1)/2]; %.*(tanh(footGroundReactionsRMC(:,1)) + 1)/2
propulsiveForce = [footGroundReactionsRMC(:,1),...
    footGroundReactionsLMC(:,1)]; %.*(tanh(footGroundReactionsRMC(:,1)) + 1)/2

qError = q - x;
% qpppError = qppp - xppp;
grfError = [grfr-footGroundReactionsRMC grfl-footGroundReactionsLMC];
% grfError = [grfr-grfr grfl-grfl];

torqueError = torques-idTorques;
neuralCommandError = [baseNeuralCommands_r - neuralCommands_r,baseNeuralCommands_l - neuralCommands_l];
% toeAngleError = x(:,[13,20]) - q(:,[13,20]);

[handPos, ~, ~, ~] = opensimGetBodyPosVel_OMP(t,[x xp],input.auxdata.handForceLoc,input.auxdata.handBodyMat);
handPosError = [onesCol*input.auxdata.posOnBarR - handPos(:,1:3),onesCol*input.auxdata.posOnBarL - handPos(:,4:6)];


phaseout(1).path = ([residualLoads(:,1:6), ... %Pelvis residuals
    torques(:,input.auxdata.whichLoadsEmg)-etmMoments, ...
    qError, ...
    grfError(:,[1:3 7:9]),grfError(:,[4:6 10:12]), ...
    handPosError, ...
    neuralCommandError]-...
    ones(nPts,1)*(input.auxdata.pathMax+input.auxdata.pathMin)/2)...
    ./(ones(nPts,1)*(input.auxdata.pathMax-input.auxdata.pathMin));


% Set integrand
% phaseout(1).integrand = [[qError...
%     grfError(:,[1:3 7:9]) ...
%     grfError(:,[4:6 10:12]) ...
%     torqueError(:,[1:6,8:13])... % Old: [1:6,8:13,15:end] We don't want to track arms and back
%     neuralCommandError ...
%     xppp].^2 ...
%     propulsiveForce];
phaseout(1).integrand = [xppp.^2 ...
    propulsiveForce];

% Scale integrand
% phaseout.integrand(:,1:96) = (phaseout.integrand(:,1:96))./(onesCol*(input.auxdata.integralMax(1:96)-input.auxdata.integralMin(1:96)));
% phaseout.integrand(:,97:98) = (phaseout.integrand(:,97:98) - ...
%     ones(nPts,1)*(input.auxdata.integralMax(97:98)+input.auxdata.integralMin(97:98))/2)./(ones(nPts,1)*(input.auxdata.integralMax(97:98)-input.auxdata.integralMin(97:98)));
phaseout.integrand(:,1:31) = (phaseout.integrand(:,1:31))./(onesCol*(input.auxdata.integralMax(1:31)-input.auxdata.integralMin(1:31)));
phaseout.integrand(:,32:33) = (phaseout.integrand(:,32:33) - ...
    ones(nPts,1)*(input.auxdata.integralMax(32:33)+input.auxdata.integralMin(32:33))/2)./(ones(nPts,1)*(input.auxdata.integralMax(32:33)-input.auxdata.integralMin(32:33)));


% phaseout.EtMmoments = EtMmoments;
% Set more outputs for post-simulation analyses
phaseout.mkrPos = mkrPos;
phaseout.markers = mkrPosReorder;
phaseout.torques = torques;
phaseout.etmMoments = etmMoments;
phaseout.muscForce = muscForce;
phaseout.actsWithStim = actsWithStim;

if max(max(isnan(phaseout.path))) || max(max(isinf(phaseout.path))) || ~min(min(isreal(phaseout.path)))
    keyboard
end
if max(max(isnan(phaseout.dynamics))) || max(max(isinf(phaseout.dynamics))) || ~min(min(isreal(phaseout.dynamics)))
    keyboard
end
if max(max(isnan(phaseout.integrand))) || max(max(isinf(phaseout.integrand))) || ~min(min(isreal(phaseout.integrand)))
    keyboard
end

end


%% Function Definitions
function act = freglyStimActivation(t,tAct,tDeact,tOn,tOff)
% Form activations using exponentials using logical indexing 
act = zeros(length(t),1);
act(t >= tOn) = 1-exp(-(1/tAct)*(t(t>=tOn)-tOn));
act(t >= tOff) = exp(-(1/tDeact)*(t(t>=tOff)-tOff));
end

function act = freglyStimActivationOverflow(t,tAct,tDeact,tOn,tOff)
if tOff > t(end)
    tOff = t(end);
end

% Form activations using exponentials using logical indexing 
act = zeros(length(t),1);
act(t >= tOn) = 1-exp(-(1/tAct)*(t(t>=tOn)-tOn));
act(t >= tOff) = exp(-(1/tDeact)*(t(t>=tOff)-tOff));
act(t <= tOn) = exp(-(1/tDeact)*(t(t <= tOn)-(tOff-t(end))));

end

function act = stimActivation(t,params,tOn,tOff)
c1 = params(1);
c2 = params(2);
c3 = params(3);
c4 = params(4);
tOnOffset = params(5);
tOnOffset2 = params(6);
tOffOffset = params(7);
tOffOffset2 = params(8);
act = (tanh(c1*(t-tOn-tOnOffset)) - tanh(c2*(t-tOff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-tOn-tOnOffset2))+exp(2*c1*(-t+tOn+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-tOff-tOffOffset2))+exp(2*c2*(-t+tOff+tOffOffset2))))/2;

end

function act = stimActivationOverflow(t,params,tOn,tOff)
c1 = params(1);
c2 = params(2);
c3 = params(3);
c4 = params(4);
tOnOffset = params(5);
tOnOffset2 = params(6);
tOffOffset = params(7);
tOffOffset2 = params(8);
act = (tanh(c1*(t-tOn-tOnOffset)) - tanh(c2*(t-tOff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-tOn-tOnOffset2))+exp(2*c1*(-t+tOn+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-tOff-tOffOffset2))+exp(2*c2*(-t+tOff+tOffOffset2))))/2 + ...
    (1-tanh(c2*(t-tOff-tOffOffset+t(end))) + ...
    c4./(exp(2*c2*(t-tOff-tOffOffset2+t(end)))+exp(2*c2*(-t+tOff+tOffOffset2-t(end)))))/2;

end

function act = stimActivationOverflow_simple(t,params,tOn,tOff)
c1 = params(1);
c2 = params(2);
tOnOffset = params(3);
tOffOffset = params(4);
act = (tanh(c1*(t-tOn-tOnOffset)) - tanh(c2*(t-tOff-tOffOffset)))/2 + ...
    (1-tanh(c2*(t-tOff-tOffOffset+t(end))))/2;

end

function actsp = FreglyActModel(muscParams)

onescol = ones(muscParams.nframes,1);

c1 = 1./muscParams.tact-1./(4*muscParams.tact);
c2 = 1./(4*muscParams.tact);
b = 10;

actsp = ((onescol*c1).*muscParams.excitations.*(0.5*tanh(b*(muscParams.excitations-muscParams.activations))+0.5)+onescol*c2).*(muscParams.excitations-muscParams.activations);

% old He model
% actsp = ((onescol*c1).*muscParams.excitations+onescol*c2).*(muscParams.excitations-muscParams.activations);
end

function Mat = BuildEtMMatrices(JAnglesAll, JVelsAll)

onesCol = ones(size(JAnglesAll(:,1)));
zerosMat = zeros(length(JAnglesAll(:,1)),3);

JAngles = JAnglesAll(:,1:6);
JVels = JVelsAll(:,1:6);

HipFEMat = [JAngles(:,1) JAngles(:,1).^2 JAngles(:,1).^3];
HipAAMat = [JAngles(:,2) JAngles(:,2).^2 JAngles(:,2).^3];
HipIEMat = [JAngles(:,3) JAngles(:,3).^2 JAngles(:,3).^3];
InteractionFEAA = [JAngles(:,1).*JAngles(:,2) JAngles(:,1).^2.*JAngles(:,2) JAngles(:,1).*JAngles(:,2).^2];
InteractionIEAA = [JAngles(:,3).*JAngles(:,2) JAngles(:,3).^2.*JAngles(:,2) JAngles(:,3).*JAngles(:,2).^2];
InteractionFEIE = [JAngles(:,1).*JAngles(:,3) JAngles(:,1).^2.*JAngles(:,3) JAngles(:,1).*JAngles(:,3).^2];
KneeMat = [JAngles(:,4) JAngles(:,4).^2 JAngles(:,4).^3];
AnkleMat = [JAngles(:,5) JAngles(:,5).^2 JAngles(:,5).^3];
SubtalarMat = [JAngles(:,6) JAngles(:,6).^2 JAngles(:,6).^3];
InteractionAnkleSub = [JAngles(:,5).*JAngles(:,6) JAngles(:,5).^2.*JAngles(:,6) JAngles(:,5).*JAngles(:,6).^2];

HipFEVelMat = [JVels(:,1) 2*JVels(:,1).*JAngles(:,1) 3*JVels(:,1).*JAngles(:,1).^2];
HipAAVelMat = [JVels(:,2) 2*JVels(:,2).*JAngles(:,2) 3*JVels(:,2).*JAngles(:,2).^2];
HipIEVelMat = [JVels(:,3) 2*JVels(:,3).*JAngles(:,3) 3*JVels(:,3).*JAngles(:,3).^2];
InteractionFEAAVel = [JVels(:,1).*JAngles(:,2)+JAngles(:,1).*JVels(:,2) 2*JAngles(:,1).*JVels(:,1).*JAngles(:,2)+JAngles(:,1).^2.*JVels(:,2) JVels(:,1).*JAngles(:,2).^2+2*JAngles(:,1).*JVels(:,2).*JAngles(:,2)];
InteractionIEAAVel = [JVels(:,3).*JAngles(:,2)+JAngles(:,3).*JVels(:,2) 2*JAngles(:,3).*JVels(:,3).*JAngles(:,2)+JAngles(:,3).^2.*JVels(:,2) JVels(:,3).*JAngles(:,2).^2+2*JAngles(:,3).*JVels(:,2).*JAngles(:,2)];
InteractionFEIEVel = [JVels(:,1).*JAngles(:,3)+JAngles(:,1).*JVels(:,3) 2*JAngles(:,1).*JVels(:,1).*JAngles(:,3)+JAngles(:,1).^2.*JVels(:,3) JVels(:,1).*JAngles(:,3).^2+2*JAngles(:,1).*JVels(:,3).*JAngles(:,3)];
KneeVelMat = [JVels(:,4) 2*JVels(:,4).*JAngles(:,4) 3*JVels(:,4).*JAngles(:,4).^2];
AnkleVelMat = [JVels(:,5) 2*JVels(:,5).*JAngles(:,5) 3*JVels(:,5).*JAngles(:,5).^2];
SubtalarVelMat = [JVels(:,6) 2*JVels(:,6).*JAngles(:,6) 3*JVels(:,6).*JAngles(:,6).^2];
InteractionAnkleSubVel = [JVels(:,5).*JAngles(:,6)+JAngles(:,5).*JVels(:,6) 2*JAngles(:,5).*JVels(:,5).*JAngles(:,6)+JAngles(:,5).^2.*JVels(:,6) JVels(:,5).*JAngles(:,6).^2+2*JAngles(:,5).*JVels(:,6).*JAngles(:,6)];

HipFEMAMat = -[onesCol 2*JAngles(:,1) 3*JAngles(:,1).^2];
HipAAMAMat = -[onesCol 2*JAngles(:,2) 3*JAngles(:,2).^2];
InteractionFEAAMA_FE = -[JAngles(:,2) 2*JAngles(:,1).*JAngles(:,2) JAngles(:,2).^2];
InteractionFEAAMA_AA = -[JAngles(:,1) JAngles(:,1).^2 2*JAngles(:,1).*JAngles(:,2)];
InteractionFEIEMA_FE = -[JAngles(:,3) 2*JAngles(:,1).*JAngles(:,3) JAngles(:,3).^2];
InteractionIEAAMA_AA = -[JAngles(:,3) JAngles(:,3).^2 2*JAngles(:,3).*JAngles(:,2)];
KneeMAMat = -[onesCol 2*JAngles(:,4) 3*JAngles(:,4).^2];
AnkleMAMat = -[onesCol 2*JAngles(:,5) 3*JAngles(:,5).^2];
SubtalarMAMat = -[onesCol 2*JAngles(:,6) 3*JAngles(:,6).^2];
InteractionAnkleMA = -[JAngles(:,6) 2*JAngles(:,5).*JAngles(:,6) JAngles(:,6).^2];
InteractionSubMA = -[JAngles(:,5) JAngles(:,5).^2 2*JAngles(:,5).*JAngles(:,6)];

Mat{1} = [onesCol HipFEMat HipAAMat HipIEMat InteractionFEAA InteractionIEAA InteractionFEIE;...
    0*onesCol HipFEVelMat HipAAVelMat HipIEVelMat InteractionFEAAVel InteractionIEAAVel InteractionFEIEVel;...
    0*onesCol HipFEMAMat zerosMat zerosMat InteractionFEAAMA_FE zerosMat InteractionFEIEMA_FE;...
    0*onesCol zerosMat HipAAMAMat zerosMat InteractionFEAAMA_AA InteractionIEAAMA_AA zerosMat];
Mat{2} = [onesCol HipFEMat HipAAMat HipIEMat KneeMat InteractionFEAA InteractionIEAA InteractionFEIE;...
    0*onesCol HipFEVelMat HipAAVelMat HipIEVelMat KneeVelMat InteractionFEAAVel InteractionIEAAVel InteractionFEIEVel;...
    0*onesCol HipFEMAMat zerosMat zerosMat zerosMat InteractionFEAAMA_FE zerosMat InteractionFEIEMA_FE;...
    0*onesCol zerosMat HipAAMAMat zerosMat zerosMat InteractionFEAAMA_AA InteractionIEAAMA_AA zerosMat;...
    0*onesCol zerosMat zerosMat zerosMat KneeMAMat zerosMat zerosMat zerosMat];
Mat{3} = [onesCol KneeMat;...
    0*onesCol KneeVelMat;...
    0*onesCol KneeMAMat];
Mat{4} = [onesCol KneeMat AnkleMat SubtalarMat InteractionAnkleSub;...
    0*onesCol KneeVelMat AnkleVelMat SubtalarVelMat InteractionAnkleSubVel;...
    0*onesCol KneeMAMat zerosMat zerosMat zerosMat;...
    0*onesCol zerosMat AnkleMAMat zerosMat InteractionAnkleMA;...
    0*onesCol zerosMat zerosMat SubtalarMAMat InteractionSubMA];
Mat{5} = [onesCol AnkleMat SubtalarMat InteractionAnkleSub;...
    0*onesCol AnkleVelMat SubtalarVelMat InteractionAnkleSubVel;...
    0*onesCol AnkleMAMat zerosMat InteractionAnkleMA;...
    0*onesCol zerosMat SubtalarMAMat InteractionSubMA];

JAngles = JAnglesAll(:,7:12);
JVels = JVelsAll(:,7:12);

HipFEMat = [JAngles(:,1) JAngles(:,1).^2 JAngles(:,1).^3];
HipAAMat = [JAngles(:,2) JAngles(:,2).^2 JAngles(:,2).^3];
HipIEMat = [JAngles(:,3) JAngles(:,3).^2 JAngles(:,3).^3];
InteractionFEAA = [JAngles(:,1).*JAngles(:,2) JAngles(:,1).^2.*JAngles(:,2) JAngles(:,1).*JAngles(:,2).^2];
InteractionIEAA = [JAngles(:,3).*JAngles(:,2) JAngles(:,3).^2.*JAngles(:,2) JAngles(:,3).*JAngles(:,2).^2];
InteractionFEIE = [JAngles(:,1).*JAngles(:,3) JAngles(:,1).^2.*JAngles(:,3) JAngles(:,1).*JAngles(:,3).^2];
KneeMat = [JAngles(:,4) JAngles(:,4).^2 JAngles(:,4).^3];
AnkleMat = [JAngles(:,5) JAngles(:,5).^2 JAngles(:,5).^3];
SubtalarMat = [JAngles(:,6) JAngles(:,6).^2 JAngles(:,6).^3];
InteractionAnkleSub = [JAngles(:,5).*JAngles(:,6) JAngles(:,5).^2.*JAngles(:,6) JAngles(:,5).*JAngles(:,6).^2];

HipFEVelMat = [JVels(:,1) 2*JVels(:,1).*JAngles(:,1) 3*JVels(:,1).*JAngles(:,1).^2];
HipAAVelMat = [JVels(:,2) 2*JVels(:,2).*JAngles(:,2) 3*JVels(:,2).*JAngles(:,2).^2];
HipIEVelMat = [JVels(:,3) 2*JVels(:,3).*JAngles(:,3) 3*JVels(:,3).*JAngles(:,3).^2];
InteractionFEAAVel = [JVels(:,1).*JAngles(:,2)+JAngles(:,1).*JVels(:,2) 2*JAngles(:,1).*JVels(:,1).*JAngles(:,2)+JAngles(:,1).^2.*JVels(:,2) JVels(:,1).*JAngles(:,2).^2+2*JAngles(:,1).*JVels(:,2).*JAngles(:,2)];
InteractionIEAAVel = [JVels(:,3).*JAngles(:,2)+JAngles(:,3).*JVels(:,2) 2*JAngles(:,3).*JVels(:,3).*JAngles(:,2)+JAngles(:,3).^2.*JVels(:,2) JVels(:,3).*JAngles(:,2).^2+2*JAngles(:,3).*JVels(:,2).*JAngles(:,2)];
InteractionFEIEVel = [JVels(:,1).*JAngles(:,3)+JAngles(:,1).*JVels(:,3) 2*JAngles(:,1).*JVels(:,1).*JAngles(:,3)+JAngles(:,1).^2.*JVels(:,3) JVels(:,1).*JAngles(:,3).^2+2*JAngles(:,1).*JVels(:,3).*JAngles(:,3)];
KneeVelMat = [JVels(:,4) 2*JVels(:,4).*JAngles(:,4) 3*JVels(:,4).*JAngles(:,4).^2];
AnkleVelMat = [JVels(:,5) 2*JVels(:,5).*JAngles(:,5) 3*JVels(:,5).*JAngles(:,5).^2];
SubtalarVelMat = [JVels(:,6) 2*JVels(:,6).*JAngles(:,6) 3*JVels(:,6).*JAngles(:,6).^2];
InteractionAnkleSubVel = [JVels(:,5).*JAngles(:,6)+JAngles(:,5).*JVels(:,6) 2*JAngles(:,5).*JVels(:,5).*JAngles(:,6)+JAngles(:,5).^2.*JVels(:,6) JVels(:,5).*JAngles(:,6).^2+2*JAngles(:,5).*JVels(:,6).*JAngles(:,6)];

HipFEMAMat = -[onesCol 2*JAngles(:,1) 3*JAngles(:,1).^2];
HipAAMAMat = -[onesCol 2*JAngles(:,2) 3*JAngles(:,2).^2];
InteractionFEAAMA_FE = -[JAngles(:,2) 2*JAngles(:,1).*JAngles(:,2) JAngles(:,2).^2];
InteractionFEAAMA_AA = -[JAngles(:,1) JAngles(:,1).^2 2*JAngles(:,1).*JAngles(:,2)];
InteractionFEIEMA_FE = -[JAngles(:,3) 2*JAngles(:,1).*JAngles(:,3) JAngles(:,3).^2];
InteractionIEAAMA_AA = -[JAngles(:,3) JAngles(:,3).^2 2*JAngles(:,3).*JAngles(:,2)];
KneeMAMat = -[onesCol 2*JAngles(:,4) 3*JAngles(:,4).^2];
AnkleMAMat = -[onesCol 2*JAngles(:,5) 3*JAngles(:,5).^2];
SubtalarMAMat = -[onesCol 2*JAngles(:,6) 3*JAngles(:,6).^2];
InteractionAnkleMA = -[JAngles(:,6) 2*JAngles(:,5).*JAngles(:,6) JAngles(:,6).^2];
InteractionSubMA = -[JAngles(:,5) JAngles(:,5).^2 2*JAngles(:,5).*JAngles(:,6)];

Mat{6} = [onesCol HipFEMat HipAAMat HipIEMat InteractionFEAA InteractionIEAA InteractionFEIE;...
    0*onesCol HipFEVelMat HipAAVelMat HipIEVelMat InteractionFEAAVel InteractionIEAAVel InteractionFEIEVel;...
    0*onesCol HipFEMAMat zerosMat zerosMat InteractionFEAAMA_FE zerosMat InteractionFEIEMA_FE;...
    0*onesCol zerosMat HipAAMAMat zerosMat InteractionFEAAMA_AA InteractionIEAAMA_AA zerosMat];
Mat{7} = [onesCol HipFEMat HipAAMat HipIEMat KneeMat InteractionFEAA InteractionIEAA InteractionFEIE;...
    0*onesCol HipFEVelMat HipAAVelMat HipIEVelMat KneeVelMat InteractionFEAAVel InteractionIEAAVel InteractionFEIEVel;...
    0*onesCol HipFEMAMat zerosMat zerosMat zerosMat InteractionFEAAMA_FE zerosMat InteractionFEIEMA_FE;...
    0*onesCol zerosMat HipAAMAMat zerosMat zerosMat InteractionFEAAMA_AA InteractionIEAAMA_AA zerosMat;...
    0*onesCol zerosMat zerosMat zerosMat KneeMAMat zerosMat zerosMat zerosMat];
Mat{8} = [onesCol KneeMat;...
    0*onesCol KneeVelMat;...
    0*onesCol KneeMAMat];
Mat{9} = [onesCol KneeMat AnkleMat SubtalarMat InteractionAnkleSub;...
    0*onesCol KneeVelMat AnkleVelMat SubtalarVelMat InteractionAnkleSubVel;...
    0*onesCol KneeMAMat zerosMat zerosMat zerosMat;...
    0*onesCol zerosMat AnkleMAMat zerosMat InteractionAnkleMA;...
    0*onesCol zerosMat zerosMat SubtalarMAMat InteractionSubMA];
Mat{10} = [onesCol AnkleMat SubtalarMat InteractionAnkleSub;...
    0*onesCol AnkleVelMat SubtalarVelMat InteractionAnkleSubVel;...
    0*onesCol AnkleMAMat zerosMat InteractionAnkleMA;...
    0*onesCol zerosMat SubtalarMAMat InteractionSubMA];
end

function [moments,muscForce,vmt] = calcMoments(muscParams)

nPts = muscParams.nPts;
alpha = muscParams.pennAngle;
% HipMA = optParams.HipMA;
% HipAAMA = optParams.HipAAMA;
% KneeMA = optParams.KneeMA;
% AnkleMA = optParams.AnkleMA;
% SubtalarMA = optParams.SubtalarMA;
% Lmt = optParams.Lmt;
% Vmt = optParams.Vmt;
onesCol = ones(nPts,1);
Mat = muscParams.EtMMats;
muscRef = muscParams.muscRef;
nMusc = length(muscRef);
coefs = muscParams.etmCoefs;

% MACurvatureOffset = optParams.MACurvatureOffset;
lts = muscParams.lts;
lmo = muscParams.lmo;
fMax = muscParams.fMax;
a = muscParams.activations;

hipMA = zeros(nPts,nMusc);
hipAAMA = zeros(nPts,nMusc);
kneeMA = zeros(nPts,nMusc);
ankleMA = zeros(nPts,nMusc);
subtalarMA = zeros(nPts,nMusc);
lmt = zeros(nPts,nMusc);
vmt = zeros(nPts,nMusc);

Ref1 = 1:nPts;
Ref2 = nPts+1:2*nPts;
Ref3 = 2*nPts+1:3*nPts;
Ref4 = 3*nPts+1:4*nPts;
Ref5 = 4*nPts+1:5*nPts;
for i = 1:nMusc
    switch muscRef(i)
        case 1
            Vec = Mat{1}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            hipMA(:,i) = Vec(Ref3);
            hipAAMA(:,i) = Vec(Ref4);
        case 2
            Vec = Mat{2}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            hipMA(:,i) = Vec(Ref3);
            hipAAMA(:,i) = Vec(Ref4);
            kneeMA(:,i) = Vec(Ref5);
        case 3
            Vec = Mat{3}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            kneeMA(:,i) = Vec(Ref3);
        case 4
            Vec = Mat{4}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            kneeMA(:,i) = Vec(Ref3);
            ankleMA(:,i) = Vec(Ref4);
            subtalarMA(:,i) = Vec(Ref5);
        case 5
            Vec = Mat{5}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            ankleMA(:,i) = Vec(Ref3);
            subtalarMA(:,i) = Vec(Ref4);
        case 6
            Vec = Mat{6}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            hipMA(:,i) = Vec(Ref3);
            hipAAMA(:,i) = Vec(Ref4);
        case 7
            Vec = Mat{7}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            hipMA(:,i) = Vec(Ref3);
            hipAAMA(:,i) = Vec(Ref4);
            kneeMA(:,i) = Vec(Ref5);
        case 8
            Vec = Mat{8}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            kneeMA(:,i) = Vec(Ref3);
        case 9
            Vec = Mat{9}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            kneeMA(:,i) = Vec(Ref3);
            ankleMA(:,i) = Vec(Ref4);
            subtalarMA(:,i) = Vec(Ref5);
        case 10
            Vec = Mat{10}*coefs{i};
            lmt(:,i) = Vec(Ref1);
            vmt(:,i) = Vec(Ref2);
            ankleMA(:,i) = Vec(Ref3);
            subtalarMA(:,i) = Vec(Ref4);
    end
end

lmtilda = (lmt-onesCol*lts)./(onesCol*(lmo.*cos(alpha)));
vmtilda = vmt./(10*onesCol*(lmo.*cos(alpha)));

passiveF = onesCol*(fMax.*cos(alpha)).*passiveForce(lmtilda);
muscForce = onesCol*(fMax.*cos(alpha)).*a.*FLCurve(lmtilda).*FVCurve(vmtilda)+passiveF;

muscMoments(:,:,1) = hipMA(:,1:(nMusc/2)).*muscForce(:,1:(nMusc/2));
muscMoments(:,:,2) = hipAAMA(:,1:(nMusc/2)).*muscForce(:,1:(nMusc/2));
muscMoments(:,:,3) = kneeMA(:,1:(nMusc/2)).*muscForce(:,1:(nMusc/2));
muscMoments(:,:,4) = ankleMA(:,1:(nMusc/2)).*muscForce(:,1:(nMusc/2));
muscMoments(:,:,5) = subtalarMA(:,1:(nMusc/2)).*muscForce(:,1:(nMusc/2));
muscMoments(:,:,6) = hipMA(:,(nMusc/2+1):nMusc).*muscForce(:,(nMusc/2+1):nMusc);
muscMoments(:,:,7) = hipAAMA(:,(nMusc/2+1):nMusc).*muscForce(:,(nMusc/2+1):nMusc);
muscMoments(:,:,8) = kneeMA(:,(nMusc/2+1):nMusc).*muscForce(:,(nMusc/2+1):nMusc);
muscMoments(:,:,9) = ankleMA(:,(nMusc/2+1):nMusc).*muscForce(:,(nMusc/2+1):nMusc);
muscMoments(:,:,10) = subtalarMA(:,(nMusc/2+1):nMusc).*muscForce(:,(nMusc/2+1):nMusc);

moments = permute(sum(muscMoments,2),[1 3 2]);
end

function FMtilda = FLCurve(lMtilda)

b11 = 0.814483478343008;
b21 = 1.055033428970575;
b31 = 0.162384573599574;
b41 = 0.063303448465465;
b12 = 0.433004984392647;
b22 = 0.716775413397760;
b32 = -0.029947116970696;
b42 = 0.200356847296188;
b13 = 0.1;
b23 = 1;
b33 = 0.5*sqrt(0.5);
b43 = 0;

FMtilda = b11*exp(-0.5*(lMtilda-b21).^2./(b31+b41*lMtilda).^2)...
    +b12*exp(-0.5*(lMtilda-b22).^2./(b32+b42*lMtilda).^2)+...
    b13*exp(-0.5*(lMtilda-b23).^2./(b33+b43*lMtilda).^2);
end

function FMtilda = FVCurve(vMtilda)
% Coefficients determined manually

d1 = -8.665833008200368;
d2 = 7.623504943764548;
d3 = 3.565407127548163;
d4 = 0.968466940650055;
d5 = -0.394453556647392;
d6 = 8.169548497993912;

FMtilda = d1+d2*atan(d3+d4*atan(d5+d6*vMtilda));
end

function PFtilda = passiveForce(lMtilda)

c1 = 3.384513491001194;
c2 = -74.090638944399615;
c3 = -2.443789456896774;

PFtilda = c1*exp(c2*(exp(lMtilda)).^c3);
end
