% Make Plots for Dissertation
clear all; close all;
dirOrig = cd;

% Load all of the results
% No stim
cd('..\11Symmetry Weighting');
load('output_symmetryWeighting_ipopt_v5pt0.mat');
noStimOutput = output;
noStimSolution = output.result.solution;

% Fixed time, free stim mag
cd('..\12pt0TA Gas Stim Fixed Timing');
load('output_taGasStimFixedTime_ipopt_v6pt16.mat'); % Has extra integral terms
fixedTimeOutput = output;
fixedTimeSolution = output.result.solution;

% All free stim params
cd('..\12pt5TA Gas Stim Optimization');
load('output_taGasStimOpt_ipopt_v11pt12.mat');
freeTimeOutput = output;
freeTimeSolution = output.result.solution;

% Choose stim muscles
cd('..\13Choose Stim Muscles');
load('output_chooseStimMuscles_ipopt_v5pt6.mat');
chooseMusclesOutput = output;
chooseMusclesSolution = output.result.solution;

% New muscles
cd('..\14New Muscles Stim');
load('output_newMusclesStimOpt_ipopt_v1pt7.mat');
newMusclesOutput = output;
newMusclesSolution = output.result.solution;

% Experimental
cd('..\SetupFiles');
exp.auxdata = SetupOpenSimOptimalTracking(false,true);

cd(dirOrig);


% Unscale everything
% No Stim
input = noStimSolution;
input.auxdata = noStimOutput.result.setup.auxdata;
noStim.auxdata = input.auxdata;
onesCol = ones(length(noStimSolution.phase.time),1);
noStim.time = (input.phase.time)*(input.auxdata.tMax-input.auxdata.tMin)+(input.auxdata.tMax+input.auxdata.tMin)/2;
noStim.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;
noStim.control = (input.phase.control).*(onesCol*(input.auxdata.controlMax-input.auxdata.controlMin))+onesCol*(input.auxdata.controlMax+input.auxdata.controlMin)/2;

% Fixed Time
input = fixedTimeSolution;
input.auxdata = fixedTimeOutput.result.setup.auxdata;
fixedTime.auxdata = input.auxdata;
onesCol = ones(length(fixedTimeSolution.phase.time),1);
fixedTime.time = input.phase.time*input.auxdata.timeScale;
fixedTime.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;
fixedTime.control = (input.phase.control).*(onesCol*(input.auxdata.controlMax-input.auxdata.controlMin))+onesCol*(input.auxdata.controlMax+input.auxdata.controlMin)/2;
fixedTime.parameter = input.parameter(1,:).*input.auxdata.paramScale;

% Free Time
input = freeTimeSolution;
input.auxdata = freeTimeOutput.result.setup.auxdata;
freeTime.auxdata = input.auxdata;
onesCol = ones(length(freeTimeSolution.phase.time),1);
freeTime.time = input.phase.time*input.auxdata.timeScale;
freeTime.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;
freeTime.control = (input.phase.control).*(onesCol*(input.auxdata.controlMax-input.auxdata.controlMin))+onesCol*(input.auxdata.controlMax+input.auxdata.controlMin)/2;
freeTime.parameter = input.parameter(1,:).*input.auxdata.paramScale;

% Choose Stim Muscles
input = chooseMusclesSolution;
input.auxdata = chooseMusclesOutput.result.setup.auxdata;
chooseMuscles.auxdata = input.auxdata;
onesCol = ones(length(chooseMusclesSolution.phase.time),1);
chooseMuscles.time = input.phase.time*input.auxdata.timeScale;
chooseMuscles.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;
chooseMuscles.control = (input.phase.control).*(onesCol*(input.auxdata.controlMax-input.auxdata.controlMin))+onesCol*(input.auxdata.controlMax+input.auxdata.controlMin)/2;
chooseMuscles.parameter = input.parameter(1,:).*input.auxdata.paramScale;

% New Muscles
input = newMusclesSolution;
input.auxdata = newMusclesOutput.result.setup.auxdata;
newMuscles.auxdata = input.auxdata;
onesCol = ones(length(newMusclesSolution.phase.time),1);
newMuscles.time = input.phase.time*input.auxdata.timeScale;
newMuscles.state = (input.phase.state).*(onesCol*(input.auxdata.stateMax-input.auxdata.stateMin))+onesCol*(input.auxdata.stateMax+input.auxdata.stateMin)/2;
newMuscles.control = (input.phase.control).*(onesCol*(input.auxdata.controlMax-input.auxdata.controlMin))+onesCol*(input.auxdata.controlMax+input.auxdata.controlMin)/2;
newMuscles.parameter = input.parameter(1,:).*input.auxdata.paramScale;


% Make Plots
plotCoords = false;
if plotCoords
% Coordinates
labels = {'Experimental','No Stim','TA-Gas Fixed Time','TA-Gas Free Time','New Muscles'};
for i = 1:20
    figure;
    if ismember(i,4:6)
        midpoint = ((max(noStim.auxdata.q(:,i)) - min(noStim.auxdata.q(:,i)))/2 + min(noStim.auxdata.q(:,i)));
        minpoint = midpoint - 0.1;
        maxpoint = midpoint + 0.1;
        plot(exp.auxdata.time/exp.auxdata.time(end), exp.auxdata.q(:,i),'k','linewidth',2);
        hold on
        plot(noStim.time/noStim.time(end), noStim.state(:,i),'r','linewidth',2)
        plot(fixedTime.time/fixedTime.time(end), fixedTime.state(:,i),'g','linewidth',2)
        plot(freeTime.time/freeTime.time(end), freeTime.state(:,i),'b','linewidth',2)
        plot(newMuscles.time/newMuscles.time(end), newMuscles.state(:,i),'c','linewidth',2)
%         plot(auxdata.time/auxdata.time(end), qGuess(:,i),'b')
        ylim([minpoint,maxpoint]);
%         title(strrep(auxdata.qLabels(i),'_',' '));
    else
        midpoint = (180/pi)*((max(noStim.auxdata.q(:,i)) - min(noStim.auxdata.q(:,i)))/2 + min(noStim.auxdata.q(:,i)));
        minpoint = midpoint - 0.7*180/pi;
        maxpoint = midpoint + 0.7*180/pi;
        plot(exp.auxdata.time/exp.auxdata.time(end), exp.auxdata.q(:,i)*180/pi,'k','linewidth',2);
        hold on
        plot(noStim.time/noStim.time(end), noStim.state(:,i)*180/pi,'r','linewidth',2)
        plot(fixedTime.time/fixedTime.time(end), fixedTime.state(:,i)*180/pi,'g','linewidth',2)
        plot(freeTime.time/freeTime.time(end), freeTime.state(:,i)*180/pi,'b','linewidth',2)
        plot(newMuscles.time/newMuscles.time(end), newMuscles.state(:,i)*180/pi,'c','linewidth',2)
        
%         plot(auxdata.time/auxdata.time(end), qGuess(:,i)*180/pi,'b')
        ylim([minpoint,maxpoint]);
%         title(strrep(auxdata.qLabels(i),'_',' '));
    end
    box off;
    ax = gca;
    set(ax,'LineWidth',2);
    set(ax,'FontSize',16);
    if ismember(i,[1,3,14:19])
        ylabel('Angle (deg)','FontSize',16);
    end
    if ismember(i,[4,5])
        ylabel('Translation (m)','FontSize',16);
    end
    if ismember(i,[5,6,9,16,10,17,12,19])
        xlabel('Cycle %','FontSize',16);
    end
    if ismember(i,[2,7,10,11])
        if i == 10
            legend(labels,'FontSize',10,'Location','northwest')
        elseif i == 7
            legend(labels,'FontSize',10,'Location','north')
        else
            legend(labels,'FontSize',10);
        end
    end
end

end


runCont = true;
if runCont
% Run Continuous Functions
% No Stim
cd('..\11Symmetry Weighting');
% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');
input = noStimSolution;
input.auxdata = noStimOutput.result.setup.auxdata;
noStim.auxdata = input.auxdata;
noStimContOut = symmetryWeightingContinuous(input);

% Fixed Time
cd('..\12pt0TA Gas Stim Fixed Timing');
% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');
input = fixedTimeSolution;
input.auxdata = fixedTimeOutput.result.setup.auxdata;
input.phase.parameter = input.parameter;
fixedTime.auxdata = input.auxdata;
fixedTimeContOut = taGasStimFixedTimeContinuous(input);

% Free Time
cd('..\12pt5TA Gas Stim Optimization');
% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');
input = freeTimeSolution;
input.auxdata = freeTimeOutput.result.setup.auxdata;
input.phase.parameter = input.parameter;
freeTime.auxdata = input.auxdata;
freeTimeContOut = taGasStimOptContinuous(input);

% Choose Stim Muscles
cd('..\13Choose Stim Muscles');
% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');
input = chooseMusclesSolution;
input.auxdata = chooseMusclesOutput.result.setup.auxdata;
input.phase.parameter = input.parameter;
chooseMuscles.auxdata = input.auxdata;
chooseMusclesContOut = chooseStimMusclesContinuous(input);

% New Muscles
cd('..\14New Muscles Stim');
% Initilize osim models in mex files
opensimPointKin_v3('Subject1_v4pt3_GPOPS.osim');
opensimID_OMP('Subject1_v4pt3_GPOPS.osim');
opensimGetBodyPosVel_OMP('Subject1_v4pt3_GPOPS.osim');
input = newMusclesSolution;
input.auxdata = newMusclesOutput.result.setup.auxdata;
input.phase.parameter = input.parameter;
newMuscles.auxdata = input.auxdata;
newMusclesContOut = newMusclesStimOptContinuous(input);
end

plotGRFs = false;
if plotGRFs
figure % Right AP Forces
plot(exp.auxdata.time/exp.auxdata.time(end),exp.auxdata.grfDataR(:,1),'k','linewidth',2)
hold on
plot(noStim.time/noStim.time(end),noStimContOut.GRFsSplit(:,1)+noStimContOut.GRFsSplit(:,4),'r','linewidth',2)
plot(fixedTime.time/fixedTime.time(end),fixedTimeContOut.GRFsSplit(:,1)+fixedTimeContOut.GRFsSplit(:,4),'g','linewidth',2)
plot(freeTime.time/freeTime.time(end),freeTimeContOut.GRFsSplit(:,1)+freeTimeContOut.GRFsSplit(:,4),'b','linewidth',2)
plot(newMuscles.time/newMuscles.time(end),newMusclesContOut.GRFsSplit(:,1)+newMusclesContOut.GRFsSplit(:,4),'c','linewidth',2)
box off;
ax = gca;
set(ax,'LineWidth',2);
set(ax,'FontSize',16);
xlabel('Cycle %','FontSize',16);
legend(labels,'FontSize',10,'Location','southeast');
    
figure % Left AP Forces
plot(exp.auxdata.time/exp.auxdata.time(end),exp.auxdata.grfDataL(:,1),'k','linewidth',2)
hold on
plot(noStim.time/noStim.time(end),noStimContOut.GRFsSplit(:,13)+noStimContOut.GRFsSplit(:,16),'r','linewidth',2)
plot(fixedTime.time/fixedTime.time(end),fixedTimeContOut.GRFsSplit(:,13)+fixedTimeContOut.GRFsSplit(:,16),'g','linewidth',2)
plot(freeTime.time/freeTime.time(end),freeTimeContOut.GRFsSplit(:,13)+freeTimeContOut.GRFsSplit(:,16),'b','linewidth',2)
plot(newMuscles.time/newMuscles.time(end),newMusclesContOut.GRFsSplit(:,13)+newMusclesContOut.GRFsSplit(:,16),'c','linewidth',2)
box off;
ax = gca;
set(ax,'LineWidth',2);
set(ax,'FontSize',16);
ylabel('Force (N)','FontSize',16);
xlabel('Cycle %','FontSize',16);
    
figure % Right Normal Forces
plot(exp.auxdata.time/exp.auxdata.time(end),exp.auxdata.grfDataR(:,2),'k','linewidth',2)
hold on
plot(noStim.time/noStim.time(end),noStimContOut.GRFsSplit(:,2)+noStimContOut.GRFsSplit(:,5),'r','linewidth',2)
plot(fixedTime.time/fixedTime.time(end),fixedTimeContOut.GRFsSplit(:,2)+fixedTimeContOut.GRFsSplit(:,5),'g','linewidth',2)
plot(freeTime.time/freeTime.time(end),freeTimeContOut.GRFsSplit(:,2)+freeTimeContOut.GRFsSplit(:,5),'b','linewidth',2)
plot(newMuscles.time/newMuscles.time(end),newMusclesContOut.GRFsSplit(:,2)+newMusclesContOut.GRFsSplit(:,5),'c','linewidth',2)
ylim([0,900]);
box off;
ax = gca;
set(ax,'LineWidth',2);
set(ax,'FontSize',16);
xlabel('Cycle %','FontSize',16);
legend(labels,'FontSize',10);
    
figure % Left Normal Forces
plot(exp.auxdata.time/exp.auxdata.time(end),exp.auxdata.grfDataL(:,2),'k','linewidth',2)
hold on
plot(noStim.time/noStim.time(end),noStimContOut.GRFsSplit(:,14)+noStimContOut.GRFsSplit(:,17),'r','linewidth',2)
plot(fixedTime.time/fixedTime.time(end),fixedTimeContOut.GRFsSplit(:,14)+fixedTimeContOut.GRFsSplit(:,17),'g','linewidth',2)
plot(freeTime.time/freeTime.time(end),freeTimeContOut.GRFsSplit(:,14)+freeTimeContOut.GRFsSplit(:,17),'b','linewidth',2)
plot(newMuscles.time/newMuscles.time(end),newMusclesContOut.GRFsSplit(:,14)+newMusclesContOut.GRFsSplit(:,17),'c','linewidth',2)
ylim([0,900]);
box off;
ax = gca;
set(ax,'LineWidth',2);
set(ax,'FontSize',16);
ylabel('Force (N)','FontSize',16);
xlabel('Cycle %','FontSize',16);

end


% Plot Joint Moments
plotJointMoments = false;
if plotJointMoments
labels = {'Experimental','No Stim','TA-Gas Fixed Time','TA-Gas Free Time','New Muscles'};
for i = 1:14
    figure;
    
    midpoint = ((max(noStim.auxdata.torques(:,i)) - min(noStim.auxdata.torques(:,i)))/2 + min(noStim.auxdata.torques(:,i)));
    minpoint = midpoint - 100;
    maxpoint = midpoint + 100;
    plot(exp.auxdata.time/exp.auxdata.time(end), exp.auxdata.torques(:,i),'k','linewidth',2);
    hold on
    plot(noStim.time/noStim.time(end), noStimContOut.torques(:,i),'r','linewidth',2)
    plot(fixedTime.time/fixedTime.time(end), fixedTimeContOut.torques(:,i),'g','linewidth',2)
    plot(freeTime.time/freeTime.time(end), freeTimeContOut.torques(:,i),'b','linewidth',2)
    plot(newMuscles.time/newMuscles.time(end), newMusclesContOut.torques(:,i),'c','linewidth',2)
    ylim([minpoint,maxpoint]);
%    title(strrep(exp.auxdata.torqueNames(i),'_',' '));
    
    box off;
    ax = gca;
    set(ax,'LineWidth',2);
    set(ax,'FontSize',16);
    if ismember(i,8:14)
        ylabel('Moment (Nm)','FontSize',16);
    end
    if ismember(i,[3,4,6,10,11,13])
        xlabel('Cycle %','FontSize',16);
    end
    if ismember(i,[1,4,5])
        if i == 5
            legend(labels,'FontSize',10,'Location','southeast')
        elseif i == 7
            legend(labels,'FontSize',10,'Location','north')
        else
            legend(labels,'FontSize',10);
        end
    end
end
    
end

% Stimulation Params
printStimParams = false;
if printStimParams
    % Fixed Time
    fixedTime_gas_mag = fixedTime.parameter(1)
    fixedTime_ta_mag = fixedTime.parameter(2)
    
    fixedTime_gas_ton = fixedTime.auxdata.gasStimTOn + fixedTime.parameter(3)
    fixedTime_ta_ton =  fixedTime.auxdata.taStimTOn + fixedTime.parameter(4)
    fixedTime_gas_pon = fixedTime_gas_ton/fixedTime.time(end)
    fixedTime_ta_pon = fixedTime_ta_ton/fixedTime.time(end)
    
    fixedTime_gas_toff = fixedTime.auxdata.gasStimTOff + fixedTime.parameter(3)
    fixedTime_ta_toff = fixedTime.auxdata.taStimTOff + fixedTime.parameter(4)
    fixedTime_gas_poff = fixedTime_gas_toff/fixedTime.time(end)
    fixedTime_ta_poff = fixedTime_ta_toff/fixedTime.time(end)
    
    % Free Time
    freeTime_gas_mag = freeTime.parameter(1)
    freeTime_ta_mag = freeTime.parameter(2)
    
    freeTime_gas_ton = freeTime.parameter(3)
    freeTime_ta_ton = freeTime.parameter(4)
    freeTime_gas_pon = freeTime_gas_ton/freeTime.time(end)
    freeTime_ta_pon = freeTime_ta_ton/freeTime.time(end)
    
    freeTime_gas_toff = freeTime_gas_ton + freeTime.parameter(5)
    freeTime_ta_toff = freeTime_ta_ton + freeTime.parameter(6)
    freeTime_gas_poff = freeTime_gas_toff/freeTime.time(end)
    freeTime_ta_poff = freeTime_ta_toff/freeTime.time(end)
    
    % New Muscles
    newMuscles_sem_mag = newMuscles.parameter(1)
    newMuscles_sol_mag = newMuscles.parameter(2)
    
    newMuscles_sem_ton = newMuscles.parameter(3)
    newMuscles_sol_ton = newMuscles.parameter(4)
    newMuscles_sem_pon = newMuscles_sem_ton/newMuscles.time(end)
    newMuscles_sol_pon = newMuscles_sol_ton/newMuscles.time(end)
    
    newMuscles_sem_toff = newMuscles_sem_ton + newMuscles.parameter(5)
    newMuscles_sol_toff = newMuscles_sol_ton + newMuscles.parameter(6)
    newMuscles_sem_poff = newMuscles_sem_toff/newMuscles.time(end)
    newMuscles_sol_poff = newMuscles_sol_toff/newMuscles.time(end)
end


getExpApIntegrals = true;
if getExpApIntegrals
    grfr = exp.auxdata.grfDataR;
    grfl = exp.auxdata.grfDataL;
    stimGrfr = exp.auxdata.stim.grfDataR;
    stimGrfl = exp.auxdata.stim.grfDataL;
    
    apIntegralR = trapz(exp.auxdata.time,grfr(:,1));
    apIntegralL = trapz(exp.auxdata.time,grfl(:,1));
    stimApIntegralR = trapz(exp.auxdata.stim.time,stimGrfr(:,1));
    stimApIntegralL = trapz(exp.auxdata.stim.time,stimGrfl(:,1));
    
    noStimIntegralR = trapz(noStim.time,noStimContOut.GRFsSplit(:,1)+noStimContOut.GRFsSplit(:,4));
    noStimIntegralL = trapz(noStim.time,noStimContOut.GRFsSplit(:,13)+noStimContOut.GRFsSplit(:,16));
    fixedTimeIntegralR = trapz(fixedTime.time,fixedTimeContOut.GRFsSplit(:,1)+fixedTimeContOut.GRFsSplit(:,4));
    fixedTimeIntegralL = trapz(fixedTime.time,fixedTimeContOut.GRFsSplit(:,13)+fixedTimeContOut.GRFsSplit(:,16));
    
    apIntSqError = (apIntegralR - apIntegralL)^2
    stimApIntSqError = (stimApIntegralR - stimApIntegralL)^2
    noStimApIntSqError = (noStimIntegralR - noStimIntegralL)^2
    fixedTimeApIntSqError = (fixedTimeIntegralR - fixedTimeIntegralL)^2
    
end